@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2013 FreeIPMI Core Team
+ * Copyright (C) 2003-2014 FreeIPMI Core Team
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -99,6 +99,16 @@ extern "C" {
  */
 #define IPMI_SENSOR_TYPE_OEM_INTEL_NMI_STATE                    0xC0
 
+/*
+ * Intel Windmill
+ * (Quanta Winterfell)
+ * (Wiwynn Windmill)
+ */
+#define IPMI_SENSOR_TYPE_OEM_INTEL_WINDMILL_ME_FW_HEALTH_SENSOR 0xDC
+
+/* Used by many sensors */
+#define IPMI_SENSOR_TYPE_OEM_INTEL_WINDMILL_GENERIC             0xC0
+
 /*
  * Intel Node Manager
  *
@@ -114,6 +124,8 @@ extern "C" {
 
 #define IPMI_SENSOR_TYPE_OEM_INTEL_NODE_MANAGER 0xDC
 
+#define IPMI_SENSOR_TYPE_OEM_INTEL_NODE_MANAGER_THERMAL_SENSOR_ON_DIMM 0x28
+
 /*******************************************
  * Inventec                                *
  *******************************************/
