@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2013 FreeIPMI Core Team
+ * Copyright (C) 2003-2014 FreeIPMI Core Team
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -82,11 +82,11 @@ FIID Template: tmpl_cmd_activate_payload_sol_rq
     { 1, "reserved3", REQUIRED, LENGTH-FIXED }
     { 1, "sol_startup_handshake", REQUIRED, LENGTH-FIXED }
     { 2, "shared_serial_alert_behavior", REQUIRED, LENGTH-FIXED }
-    { 1, "test_mode", OPTIONAL, LENGTH-FIXED }
-    { 1, "reserved4", OPTIONAL, LENGTH-FIXED }
-    { 1, "authentication_activation", OPTIONAL, LENGTH-FIXED }
-    { 1, "encryption_activation", OPTIONAL, LENGTH-FIXED }
-    { 24, "reserved5", OPTIONAL, LENGTH-FIXED }
+    { 1, "reserved4", REQUIRED, LENGTH-FIXED }
+    { 1, "test_mode", REQUIRED, LENGTH-FIXED }
+    { 1, "authentication_activation", REQUIRED, LENGTH-FIXED }
+    { 1, "encryption_activation", REQUIRED, LENGTH-FIXED }
+    { 24, "reserved5", REQUIRED, LENGTH-FIXED }
 
 Activate Payload Response
 -------------------------
